// xi.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "di.h"
#include "xi.h"
#include "ini.h"
#include <stdio.h>


bool didinit = false;
HWND hWin = NULL;
DWORD PID = NULL;

// Map internal IDs to XInput constants
const WORD buttonIDs[14] = {
	XINPUT_GAMEPAD_A,
	XINPUT_GAMEPAD_B,
	XINPUT_GAMEPAD_X,
	XINPUT_GAMEPAD_Y,
	XINPUT_GAMEPAD_LEFT_SHOULDER,
	XINPUT_GAMEPAD_RIGHT_SHOULDER,
	XINPUT_GAMEPAD_BACK,
	XINPUT_GAMEPAD_START,
	XINPUT_GAMEPAD_LEFT_THUMB,
	XINPUT_GAMEPAD_RIGHT_THUMB,
	XINPUT_GAMEPAD_DPAD_UP,
	XINPUT_GAMEPAD_DPAD_DOWN,
	XINPUT_GAMEPAD_DPAD_LEFT,
	XINPUT_GAMEPAD_DPAD_RIGHT
};

char * buttonNames[14] = {
	"A",
	"B",
	"X",
	"Y",
	"Left Shoulder",
	"Right Shoulder",
	"Back",
	"Start",
	"Left Thumb",
	"Right Thumb",
	"D-pad Up",
	"D-pad Down",
	"D-pad Left",
	"D-pad Right"
};

char * axisNames[4] = {
	"Left Analog X",
	"Left Analog Y",
	"Right Analog X",
	"Right Analog Y"
};

enum MappingType { NONE, DIGITAL, AXIS, SLIDER };
struct TriggerMap {
	int id;			// Index for the mapped button/axis/slider
	MappingType type;
};

// Axis indexes are positive or negative numbers, zero is invalid.
// All other indexer values start from zero.
int ButtonMapping[14];
TriggerMap TriggerMapping[2];
int AxisMapping[4]; // Index of axes to use. Negative index used if it needs to be inverted
int DpadPOV; // Index of POV switch to use for the D-pad

void InitConfig() {

	char	*pStr, strPath[255];
	GetModuleFileName (NULL, strPath, 255);
	pStr = strrchr(strPath, '\\');
	if (pStr != NULL)
	    *(++pStr)='\0'; 

    sprintf(strMySystemFile,"%s%s",strPath, "xbox360cemu.ini");

	char buffer[MAX_PATH];

	if (ReadFromFile("PAD1", "UseForceFeedback", buffer) > 0)
	{
		useforce = atoi(buffer);
	}
	for (int i = 0; i < 4; ++i) AxisMapping[i] = i;
	for (int i = 0; i < 14; ++i) ButtonMapping[i] = -1;
	for (int i = 0; i < 2; ++i) TriggerMapping[i].type = NONE;
	DpadPOV = -1;

	// All values start from 1 in the INI file

	for (int i=0; i<14; ++i) {
		if (ReadFromFile("PAD1", buttonNames[i], buffer) > 0) {
			int val = atoi(buffer);
			ButtonMapping[i] = val - 1;
		}
	}

	for (int i=0; i<4; ++i) {
		if (ReadFromFile("PAD1", axisNames[i], buffer) > 0) {
			int val = atoi(buffer);
			AxisMapping[i] = val;
		}
	}
	
	if (ReadFromFile("PAD1", "Left Trigger", buffer) > 0) {
		char* p = buffer;
		if (tolower(*p) == 's') { // Slider
			TriggerMapping[0].type = SLIDER;
			++p;
			TriggerMapping[0].id = atoi(p) - 1;
		} else
		if (tolower(*p) == 'a') { // Axis
			TriggerMapping[0].type = AXIS;
			++p;
			TriggerMapping[0].id = atoi(p);
		} else { // Button
			TriggerMapping[0].type = DIGITAL;
			TriggerMapping[0].id = atoi(p) - 1;
		}
	}

	if (ReadFromFile("PAD1", "Right Trigger", buffer) > 0) {
		char* p = buffer;
		if (tolower(*p) == 's') { // Slider
			TriggerMapping[1].type = SLIDER;
			++p;
			TriggerMapping[1].id = atoi(p) - 1;
		} else
		if (tolower(*p) == 'a') { // Axis
			TriggerMapping[1].type = AXIS;
			++p;
			TriggerMapping[1].id = atoi(p);
		} else { // Button
			TriggerMapping[1].type = DIGITAL;
			TriggerMapping[1].id = atoi(p) - 1;
			}
	}

	if (ReadFromFile("PAD1", "D-pad POV", buffer) > 0) {
		int val = atoi(buffer);
		DpadPOV = val - 1;
	}
}

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    return TRUE;
}

HWND GetWindowHandle(DWORD tPID)
{
	//Get first window handle
	HWND res = FindWindow(NULL,NULL);
	DWORD mPID = NULL;
	while(res != 0)
	{
		if(!GetParent(res))
		{
			GetWindowThreadProcessId(res,&mPID);
			if (mPID == tPID)
				return res;
		}
		res = GetWindow(res, GW_HWNDNEXT);
	}
}

DWORD WINAPI XInputGetState(DWORD dwUserIndex, XINPUT_STATE* pState)
{
	if (dwUserIndex == 0)
	{
		if (!didinit) 
		{
			Beep(100,100);

			while (hWin == 0)
				hWin = GetWindowHandle(GetCurrentProcessId());
			InitConfig();
			InitDirectInput(hWin, 0);
			didinit = true;
		}
		PollDevices();
		//if (KeyDown(2))
		//{
		//	Beep(100,100);
		//	SmallForce(65535);
		//}
		XINPUT_GAMEPAD &Gamepad = pState->Gamepad;

		Gamepad.wButtons = 0;
		Gamepad.bLeftTrigger = 0;
		Gamepad.bRightTrigger = 0;
		
		// Map buttons
		for (int i = 0; i < 14; ++i) {
			if ((ButtonMapping[i] >= 0)
			&&	ButtonPressed(ButtonMapping[i])
			) {
				Gamepad.wButtons |= buttonIDs[i];
			}
		}

		// Map POV to the D-pad
		if (DpadPOV >= 0) {
			int pov = POVState(DpadPOV);
			if ((pov == 7) || (pov == 0) || (pov == 1) || (pov == 8)) // Up-left, up, up-right, up (at 360 degrees) 
			{	Gamepad.wButtons |= XINPUT_GAMEPAD_DPAD_UP;		}
			if ((pov >= 1) && (pov <= 3)) // Up-right, right, down-right
			{	Gamepad.wButtons |= XINPUT_GAMEPAD_DPAD_RIGHT;	}
			if ((pov >= 3) && (pov <= 5)) // Down-right, down, down-left
			{	Gamepad.wButtons |= XINPUT_GAMEPAD_DPAD_DOWN;	}
			if ((pov >= 5) && (pov <= 7)) // Down-left, left, up-left
			{	Gamepad.wButtons |= XINPUT_GAMEPAD_DPAD_LEFT;	}
		}
		
		// Created so we can refer to each axis with an ID
		LONG axis[6] = {
			js.lX,
			js.lY,
			js.lZ,
			js.lRx,
			js.lRy,
			js.lRz
		};

		// Map triggers
		if (TriggerMapping[0].type == DIGITAL) {
			if (ButtonPressed(TriggerMapping[0].id))
			{	Gamepad.bLeftTrigger = 255;		}
		} else 
		if (TriggerMapping[0].type == AXIS) {
			Gamepad.bLeftTrigger = 128 + 0.127 *
				(TriggerMapping[0].id > 0 ?
				axis[TriggerMapping[0].id - 1] :
				-axis[-TriggerMapping[0].id - 1]);
		} else 
		if (TriggerMapping[0].type == SLIDER) {
			Gamepad.bLeftTrigger = js.rglSlider[TriggerMapping[0].id] * 0.127 + 128; 
		}
		if (TriggerMapping[1].type == DIGITAL) {
			if (ButtonPressed(TriggerMapping[1].id))
			{	Gamepad.bRightTrigger = 255;	}
		} else 
		if (TriggerMapping[1].type == AXIS) {
			Gamepad.bRightTrigger = 128 + 0.127 *
				(TriggerMapping[1].id > 0 ?
				axis[TriggerMapping[1].id - 1] :
				-axis[-TriggerMapping[1].id - 1]);
		} else 
		if (TriggerMapping[1].type == SLIDER) {
			Gamepad.bRightTrigger = js.rglSlider[TriggerMapping[1].id] * 0.127 + 128;
		}

		// Map thumbsticks
		Gamepad.sThumbLX = 32.767 *
			( AxisMapping[0] > 0 ?
			axis[AxisMapping[0] - 1] :
			-axis[-AxisMapping[0] - 1] );
		Gamepad.sThumbLY = 32.767 *
			( AxisMapping[1] > 0 ?
			axis[AxisMapping[1] - 1] :
			-axis[-AxisMapping[1] - 1] );

		Gamepad.sThumbRX = 32.767 *
			( AxisMapping[2] > 0 ?
			axis[AxisMapping[2] - 1] :
			-axis[-AxisMapping[2] - 1] );
		Gamepad.sThumbRY = 32.767 *
			( AxisMapping[3] > 0 ?
			axis[AxisMapping[3] - 1] :
			-axis[-AxisMapping[3] - 1] );

		return 0;
	}
	else
	{
		return -1;
	}
}

DWORD WINAPI XInputSetState(DWORD dwUserIndex, XINPUT_VIBRATION* pVibration)
{
	if (dwUserIndex == 0)
	{

		if (pVibration->wLeftMotorSpeed > 0)
		{
			LargeForce(pVibration->wLeftMotorSpeed);
		}
		if (pVibration->wRightMotorSpeed > 0)
		{
			SmallForce(pVibration->wRightMotorSpeed);
		}

		return 0;
	}
	//else
	//{
	//	return -1;
	//}
}

DWORD WINAPI XInputGetCapabilities(DWORD dwUserIndex, DWORD dwFlags, XINPUT_CAPABILITIES* pCapabilities)
{
	//if (dwUserIndex == 0)
	//{
		return 0;
	//}
	//else
	//{
	//	return -1;
	//}
}

void WINAPI XInputEnable(BOOL enable)
{



}

DWORD WINAPI XInputGetDSoundAudioDeviceGuids
(
    DWORD dwUserIndex,          // [in] Index of the gamer associated with the device
    GUID* pDSoundRenderGuid,    // [out] DSound device ID for render
    GUID* pDSoundCaptureGuid    // [out] DSound device ID for capture
)
{
	return 0;
}

DWORD WINAPI XInputGetBatteryInformation
(
    DWORD                       dwUserIndex,        // [in]  Index of the gamer associated with the device
    BYTE                        devType,            // [in]  Which device on this user index
    XINPUT_BATTERY_INFORMATION* pBatteryInformation // [out] Contains the level and types of batteries
)
{
	return 0;
}

DWORD WINAPI XInputGetKeystroke
(
    DWORD dwUserIndex,              // [in]  Index of the gamer associated with the device
    DWORD dwReserved,               // [in]  Reserved for future use
    PXINPUT_KEYSTROKE pKeystroke    // [out] Pointer to an XINPUT_KEYSTROKE structure that receives an input event.
)
{
	return 0;
}